/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Malte Kraus.
 *
 * Contributor(s):
 *  Malte Kraus <firefox@maltekraus.de> (Original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function adblockFilterExporter() {
  window.addEventListener("load", this._init, false);
  window.addEventListener("unload", this._uninit, false);
}
adblockFilterExporter.prototype = {
  _service: null,
  _Uploader: function() {
    var up = Components.classes["@adblockfilters.mozdev.org/uploadjob;1"]
                       .createInstance().wrappedJSObject;
    up.origWindow = window;
    up._adblockFilterExport = gAdblockFilterExport;
    up.hash = gAdblockFilterExport.hash;
    up.data = gAdblockFilterExport.data;
    up.compress();
    up.onNotUploaded = function() {
      document.documentElement.getButton("extra1").disabled = false;
    };
    up._windowDestroyed = function() {
      this.onNotUploaded = function() {};
      up.origWindow = null;
    };
    return up;
  },
  _up: null,

  uploadData: function adblockFilters__uploadData() {
    const URL = "chrome://adblockfilters/content/ftp-upload.xul";
    window.openDialog(URL, URL, "chrome", this._up = this._Uploader(), false);
  },
  _init: function adblockFilters___init() {
    (function() { // use closures to separate variables
      var service = Components.classes["@adblockfilters.mozdev.org/service;1"]
                              .getService().wrappedJSObject;
      gAdblockFilterExport._service = service;
      gAdblockFilterExport.blacklist = service.blacklist.concat([]);
    })();

    (function() {
      var oldOnChange = onChange;
      window.onChange = function onChange() {
        oldOnChange();
        var btn = document.documentElement.getButton("extra1");
        btn.disabled = !gAdblockFilterExport.dataChanged;
      };
    })();

    (function() {
      var origStopEditor = treeView.stopEditor;
      treeView.stopEditor = function abpfilteradmin__fillContext(save, blur) {
        if (this.editedRow < 0)
          return;

        var info = this.getRowInfo(this.editedRow);
        var isDummy = info && (info[0].dummy || (info[1] && info[1].dummy));
        var beforeFilter = (isDummy ? "" : info[1].text);
        var afterFilter = abp.normalizeFilter(this.editor.value);
        var filterIndex = gAdblockFilterExport.blacklist.indexOf(beforeFilter);
        origStopEditor.apply(this, arguments);
        if(save && beforeFilter && afterFilter && afterFilter != beforeFilter &&
           filterIndex != -1) {
          gAdblockFilterExport.blacklist.splice(filterIndex, 0, afterFilter);
        }
      };
    })();
    (function() {
      var origRemove = treeView.removeRow;
      treeView.removeRow = function(info) {
        var group, pattern, idx; [group, pattern] = info;
        if(pattern && typeof pattern != "string" && group.special &&
           (idx = gAdblockFilterExport.blacklist.indexOf(pattern.text)) != -1) {
          gAdblockFilterExport.blacklist.splice(idx, 1);
        }
        origRemove.apply(this, arguments);
      }
    })();
    (function() {
      var origApply = applyChanges;
      applyChanges = function() {
        gAdblockFilterExport._service.blacklist = gAdblockFilterExport.blacklist;
        origApply.apply(this, arguments);
      };
    })();
    document.getElementById("listitem-context").addEventListener("popupshowing", function(e) {
      gAdblockFilterExport.onContextShowing(e);
    }, false);

    (function() {
      document.documentElement.buttons += ",extra1";
      var btn = document.documentElement.getButton("extra1");
      btn.disabled = !gAdblockFilterExport.dataChanged;
      btn.hidden = false;

      (function() {
        var url = "chrome://adblockfilters/locale/abp-settings.properties";
        var bundleService=Components.classes["@mozilla.org/intl/stringbundle;1"]
                      .getService(Components.interfaces.nsIStringBundleService);
        var strings = bundleService.createBundle(url);
        btn.setAttribute("label", strings.GetStringFromName("upload-button"));
      })();

      btn.addEventListener("command", function(event) {
        event.stopPropagation();
        applyChanges();
        gAdblockFilterExport.uploadData();
        //window.close();
        btn.disabled = true;
      }, false);
    })();
  },
  _uninit: function() {
    if(gAdblockFilterExport._up) {
      gAdblockFilterExport._up._windowDestroyed();
    }
  },

  get data() {
    var filters = treeView.data, patterns = [];
    for(var i = 0; i < filters.length; i++) {
      if(filters[i].dummy || !filters[i].special)
        continue;
      var list = filters[i].patterns.slice().sort(sortNatural);
      for(var j = 0; j < list.length; j++) {
        var pattern = {
          text: list[j].orig.text,
          type: list[j].orig.type,
          disabled: (list[j].orig.text in treeView.disabled)
        };
        patterns.push(pattern);
      }
    }
    return this._service._serializePatternArray(patterns);
  },
  get hash() {
    return this._service.__lookupGetter__("hash").call(this);
  },
  get dataChanged() {
    return this._service.__lookupGetter__("dataChanged").call(this);
  },

  blacklist: null,

  showPreferences: function adblockFilters__showPreferences() {
    window.openDialog("chrome://adblockfilters/content/settings.xul", "adblockfilters-options",
                      "chrome,titlebar,toolbar,centerscreen,modal=0,dialog=0");
  },

  onContextShowing: function(event) {
    var selected = treeView.getSelectedInfo(), notBlacklisted = null;
    for(var i = 0; i < selected.length && !notBlacklisted; i++) {
      if(selected[i][1] && typeof selected[i][1] != "string") {
        notBlacklisted = (this.blacklist.indexOf(selected[i][1].text) == -1);
      }
    }
    if(notBlacklisted === null)
      document.getElementById("context-toggle-uploader-blacklist").hidden = true;
    else
      document.getElementById("context-toggle-uploader-blacklist").setAttribute("checked", notBlacklisted);
  },
  toggleBlacklist: function() {
    var add = document.getElementById("context-toggle-uploader-blacklist").getAttribute("checked") != "true";
    var selected = treeView.getSelectedInfo(), text, idx;
    for(var i = 0; i < selected.length; i++) {
      if(selected[i][1] && typeof selected[i][1] != "string") {
        text = selected[i][1].text;
        idx = this.blacklist.indexOf(text)
        if(idx == -1 && add)
          this.blacklist.push(text);
        else if(idx != -1 && !add)
          this.blacklist.splice(idx, 1);
      }
    }
  }
}
var gAdblockFilterExport = new adblockFilterExporter();